from enum import Enum


class NodeType(Enum):
    BARCHART = "Bar_Chart"
    CODE = "Code"
    COLUMN = "Column"
    COMMENT = "Comment"
    CONCAT = "Concat"
    CROSSJOIN = "Cross_Join"
    CUMSUM = "Cumsum"
    DATABASE = "Database"
    DATACLEANSING = "Data_Cleansing"
    DATETIMEEXTRACT = "Datetime_Extract"
    DATETIMEFORMATTER = "Datetime_Formatter"
    DESCRIBE = "Describe"
    DROPDUPLICATES = "Drop_Duplicates"
    DTYPE = "Dtype"
    FILTER = "Filter"
    FORMULA = "Formula"
    GROUPBY = "Group_By"
    INPUTDATA = "Input_Data"
    INTERROW = "Inter_Row"
    ISIN = "Isin"
    JOINMULTI = "Join_Multi"
    LINECHART = "Line_Chart"
    MELT = "Melt"
    MERGE = "Merge"
    OUTPUTDATA = "Output_Data"
    PIECHART = "Pie_Chart"
    PIVOT = "Pivot"
    SCATTERCHART = "Scatter_Chart"
    SELECT = "Select"
    SHAPE = "Shape"
    SORT = "Sort"
    SWITCH = "Switch"
    UNIQUE = "Unique"
    VALUECOUNTS = "Value_Counts"
    VOUTPUT = "V_Output"
    EXCEL = "Excel"
    EMAIL = "Email"
    IF = "If"
    WAIT = "Wait"
    DFMAKER = "DF_Maker"
    SOURCE = "Source"
